/* Copyright (c) 2004..2008 Dirk Jagdmann <doj@cubic.org>

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you
       must not claim that you wrote the original software. If you use
       this software in a product, an acknowledgment in the product
       documentation would be appreciated but is not required.

    2. Altered source versions must be plainly marked as such, and
       must not be misrepresented as being the original software.

    3. This notice may not be removed or altered from any source
       distribution. */

// $Header: /code/doj/path.hpp,v 1.25 2009/04/22 20:31:03 doj Exp $

#ifndef PATH__HPP
#define PATH__HPP

#include <stdio.h>
#include <set>
#include <string>

/// namespace which contains functions and classes from the dojlib
namespace doj
{
  /** removes useless aspects (mostly . and ..) of p
      @param p pathname
      @return the stripped version of p
  */
  std::string cleanupPath(std::string p);

  /** converts fn into an absolute pathname.
      @param fn pathname
      @return the absolute pathname of fn
  */
  std::string absFilename(std::string fn);

  /** extract directory part of a pathname. This function does not do any checks of the filesystems state.
      @param fn pathname
      @return the directory part of fn. If fn is empty return root directory. If fn does not contains any directory return fn.
  */
  std::string dirname(std::string fn);

  /** extract filename part of a pathname. This function does not do any checks of the filesystem state.
      @param fn pathname
      @return file name part of fn. We expect directories to be separated by the locally used (/,\\,:) characters.
  */
  std::string filename(std::string fn);

  /** quote a filename for command line usage.
      @param fn file name
      @return quoted fn
  */
  inline
  std::string quoteFilename(const std::string& fn)
  {
    //lint --e{953}
    std::string s="'"; s.reserve(fn.size());
    for(std::string::const_iterator i=fn.begin(), e=fn.end(); i!=e; ++i)
      switch(*i)
	{
	case '\'': s+="'\"'\"'"; break;
	default:   s+=*i; break;
	}
    s+='\'';
    return s;
  }

  /// @return true if filename exists and has any file type @param filename pathname
  bool exists(const std::string& filename);
  /// @return true if filename is a file @param filename pathname
  bool isFile(const std::string& filename);
  /// @return true if filename is an executable file @param filename pathname
  bool isExec(const std::string& filename);
  /// @return true if filename is a directory @param filename pathname
  bool isDir(const std::string& filename);
  /// @return true if filename is a character device @param filename pathname
  bool isCharDev(const std::string& filename);
  /// @return true if filename is a block device @param filename pathname
  bool isBlockDev(const std::string& filename);
  /// @return true if filename is a FIFO special file @param filename pathname
  bool isFifo(const std::string& filename);
  /// @return true if filename is a link @param filename pathname
  bool isLink(const std::string& filename);
  /// @return true if filename is a socket @param filename pathname
  bool isSocket(const std::string& filename);

  /// @return -1 upon error, file size of regular file filename @param filename pathname
  off_t fileSize(const std::string& filename);

  /** lookup access time of a file.
      @param filename pathname of file in local filesystem
      @return access time timestamp
  */
  time_t aTime(const std::string& filename);

  /** lookup creation time of a file.
      @param filename pathname of file in local filesystem
      @return creation time timestamp
  */
  time_t cTime(const std::string& filename);

  /** lookup modification time of a file.
      @param filename pathname of file in local filesystem
      @return modification time timestamp
  */
  time_t mTime(const std::string& filename);

  /// @return the url escaped representation of u @param u string
  std::string url_escape(const std::string& u);

  /// @return the unescaped string representation of u @param u string
  std::string url_unescape(const std::string& u);

  /** read a complete file.
      @param fn pathname of file in local filesystem
      @return string of fn
  */
  std::string slurpFile(const std::string& fn);

  /** read a complete stream.
      @param f pointer to FILE stream
      @return string of fn
  */
  std::string slurpFile(FILE *f);

  /** checks if fd is a valid file descriptor.
      @param fd file descriptor
      @return true if fd is valid and open
  */
  bool isValidFD(int fd);

  /** Creates a directory. Not existing parent directories are also created.
      @param dirname pathname to directory
      @param mode file permissions of newly created directories
      @return true on success.
   */
  bool mkdir(const std::string& dirname, const unsigned mode=0750);

  typedef std::set<std::string> filesInDir_t; ///< type returned by filesInDir(), which is a set of strings

  /** get list of files in a directory.
      @param dirname pathname of directory on local filesystem
      @return set of strings, each string is a filename in dirname
  */
  filesInDir_t filesInDir(const std::string& dirname);
}

#endif
